%function [InfPerSpike] = InfoPerSpike(cell1) % this expects a matrix of [spikes time] pairs, with one pair for each stimulus, place, etc, e.g. cell 1 below
%function [InfPerSpike] = InfoPerSpike(SpikesPerStimulus, TimePerStimulus) % another possible function call
% SpikesPerStimulus and TimePerStimulus are arrays of the type shown below that specify 
% the number of spikes for each stimulus and 
% the length of time in which those spikes were obtained

% The Skaggs et al (1993, NIPS) information per spike measure is very close to the sparseness a
% defined by Treves and Rolls (1991 Network 2 371-395),
% as shown in Rolls and Treves (1998, Neural Networks and Brain Function pages 311-312, .pdf available at www.oxcns.org)
% A value of a=1/N where N is the number of stimuli indicates a local or grandmother cell representation.
% A larger value for a indicates a distributed representation. 
% a=1 indicates that the firing rates are the same to all stimuli (i.e. not tuned at all).
% To calculate the sparseness a, we need the mean firing rate to each stimulus.
% 'Mutual' in 'Mutual Information' refers to the mean information calculated across stimuli.
% This program calls sparseE.m
% Here is a worked example:
format compact;
%SpikesPerStimulus = [54 159 73 100 220 150 174 186 197 161 253 1090 832 373 271 146 122 201 83 89 64 55 59 43 96 66];
%TimePerStimulus = [5.6667 17.2167 7.7667 9.7333 26.7000 19.4667 17.1500 22.7000 22.2167 20.7500 30.2000 158.6000 ...
     %129.4000 42.7667 31.5833 19.3167 15.0333 21.8000 10.6333 11.0167 7.3500 8.5833 5.4667 4.4833 7.5167 5.8500];

% Just test data on the next few lines. Normally commented out.
% SpikesPerStimulus = [30 0.001 0.001 0.001]; % used as test data
%SpikesPerStimulus = [30 20 10 0.001];  % used as test data
%TimePerStimulus = [1 1 1 1 ];  % used as test data
%SpikesPerStimulus = [1 0 0 0]; % used as test data
%TimePerStimulus = [2 2 2 2 ];  % used as test data
% cell1=[2	2
% 0.5	2
% 0	2
% 0	2];

% insular taste neuron
% cell1=[6	1
% 15	1
% 22	1
% 15	1
% 13   1];

% insular taste neuron tested with 5 stimuli
cell1=[3	1
6	1
0	1
0	1
0   1];

% cell1= [10	0.5
% 10	0.5
% 10	0.5
% 10	0.5
% 0	0.5
% 0	0.5
% 0	0.5
% 0	0.5];

SpikesPerStimulus = cell1(:, 1);
TimePerStimulus = cell1(:, 2);

% Anther set of test data for 26 stimuli:
% SpikesPerStimulus = [54 159 73 100 220 150 174 186 197 161 253 1090 832 373 271 146 122 201 83 89 64 55 59 43 96 66];
% TimePerStimulus = [5.6667 17.2167 7.7667 9.7333 26.7000 19.4667 17.1500 22.7000 22.2167 20.7500 30.2000 158.6000 ...
%      129.4000 42.7667 31.5833 19.3167 15.0333 21.8000 10.6333 11.0167 7.3500 8.5833 5.4667 4.4833 7.5167 5.8500];
     
TotalTime = sum(TimePerStimulus);
ProbPerStimulus = TimePerStimulus / TotalTime;
sum(ProbPerStimulus); % just a data check
Rate = SpikesPerStimulus ./ TimePerStimulus; % Note that the rates must be > 0.
for i = 1 : length(Rate) % Force the Rates to be > 0
    if Rate(i) <= 0
        Rate(i) = 0.001;
    end
end
MeanRate= sum(SpikesPerStimulus) / TotalTime; % It is important to calculate it this way
InfPerSpike = 0; % The Skaggs measure of information per spike
for stim = 1 : length(Rate) % over the number of stimuli
    %InfPerSpike = InfPerSpike + (ProbPerStimulus(stim) * ((Rate(stim) / MeanRate) * log2(Rate(stim) / MeanRate))); % or the following 2 lines
    InfPerStimulus = ProbPerStimulus(stim) * ((Rate(stim) / MeanRate) * log2(Rate(stim) / MeanRate)); % This is the stimulus-specific information
    InfPerSpike = InfPerSpike + InfPerStimulus; % This adds up the stimulus-specific information to produce the average information across all stimuli
    %log2(Rate(stim) / MeanRate) % debug
end
fprintf('Calculated Mutual Information per Spike =%5.3f bits per spike\n', InfPerSpike);
fprintf('Calculated Mutual Information rate      =%5.3f bits per second with MeanRate=%4.1f spikes/s\n', InfPerSpike * MeanRate, MeanRate);

% Now see how similar this value is to Chi calculated from the Sparseness
Sparseness = sparseE(Rate);
Chi = log2(1/Sparseness); % Chi is the (maximum value of) the mutual information per spike, see R&T page 312
fprintf('Sparseness=%5.3f  Maximum value for Mutual Information per Spike from sparseness (Chi)=%5.3f bits\n', Sparseness, Chi);
figure(1)
bar(Rate);
xlabel('Stimulus number');
ylabel('Rate');

% ETR added breadth of tuning (sometimes called entropy) used for taste 9 March 2020
% see Scott et al (1986) J Neurophysiol 56: 876-890.
H = 0; % Breadth of Tuning, H
SumRate = sum(Rate);
PropOfSumRate = zeros(length(Rate)); % The proportion for each stimulus of the sum of the rates
SumPlogP = 0;
for i = 1 : length(Rate)
    PropOfSumRate(i) = Rate(i) / SumRate;
    SumPlogP = SumPlogP + PropOfSumRate(i) * log10(PropOfSumRate(i));
end
% Now calc K such that H=1 when the neuron responds equally to all stimuli
RateTest= ones(length(Rate),1);
SumRateTest = sum(RateTest);
PropOfSumRateTest = zeros(length(RateTest)); % The proportion for each stimulus of the sum of the rates
SumPlogPTest = 0;
for i = 1 : length(RateTest)
    PropOfSumRateTest(i) = RateTest(i) / SumRateTest;
    SumPlogPTest = SumPlogPTest + PropOfSumRateTest(i) * log10(PropOfSumRateTest(i));
end
% Now calc K such that H=1 when the neuron responds equally to all stimuli
K = - 1 /  SumPlogPTest;
% K = 1.661; % for length(Rate) = 4
H = -K * SumPlogP; 
fprintf('H, Breadth of Tuning=%5.3f\n', H);

